@@ -7,56 +7,51 @@
 #
 # Written by Andy Wardley <abw@wardley.org>
 #
-# Copyright (C) 1997-2007 Andy Wardley.  All Rights Reserved.
+# Copyright (C) 1997-2003 Andy Wardley.  All Rights Reserved.
 # Copyright (C) 1997,1998 Canon Research Centre Europe Ltd.
-#==========================================================================
+#
+# $Id: AppConfig.pm,v 1.4 2003/04/29 09:44:51 abw Exp $
+#
+#============================================================================
 
 package AppConfig;
 
+require 5.005;
+
 use strict;
-use warnings;
-use base 'Exporter';
-our $VERSION = '1.66_03';
+use Exporter;
+use vars qw( $VERSION $AUTOLOAD @EXPORT_OK %EXPORT_TAGS );
+use base qw( Exporter );
+
+## This is the main version number for AppConfig
+## It is extracted by ExtUtils::MakeMaker and inserted in various places.
+$VERSION = '1.54';
 
 # variable expansion constants
-use constant EXPAND_NONE   => 0;
-use constant EXPAND_VAR    => 1;
-use constant EXPAND_UID    => 2;
-use constant EXPAND_ENV    => 4;
-use constant EXPAND_ALL    => EXPAND_VAR | EXPAND_UID | EXPAND_ENV;
-use constant EXPAND_WARN   => 8;
+use constant EXPAND_NONE    => 0;
+use constant EXPAND_VAR     => 1;
+use constant EXPAND_UID     => 2;
+use constant EXPAND_ENV     => 4;
+use constant EXPAND_ALL     => EXPAND_VAR | EXPAND_UID | EXPAND_ENV;
+use constant EXPAND_WARN    => 8;
 
 # argument count types
-use constant ARGCOUNT_NONE => 0;
-use constant ARGCOUNT_ONE  => 1;
-use constant ARGCOUNT_LIST => 2;
-use constant ARGCOUNT_HASH => 3;
+use constant ARGCOUNT_NONE  => 0;
+use constant ARGCOUNT_ONE   => 1;
+use constant ARGCOUNT_LIST  => 2;
+use constant ARGCOUNT_HASH  => 3;
 
 # Exporter tagsets
-our @EXPAND = qw(
-    EXPAND_NONE
-    EXPAND_VAR
-    EXPAND_UID
-    EXPAND_ENV 
-    EXPAND_ALL
-    EXPAND_WARN
-);
+my @EXPAND   = qw(EXPAND_NONE EXPAND_VAR EXPAND_UID EXPAND_ENV 
+                  EXPAND_ALL EXPAND_WARN);
+my @ARGCOUNT = qw(ARGCOUNT_NONE ARGCOUNT_ONE ARGCOUNT_LIST ARGCOUNT_HASH);
 
-our @ARGCOUNT = qw(
-    ARGCOUNT_NONE
-    ARGCOUNT_ONE
-    ARGCOUNT_LIST
-    ARGCOUNT_HASH
-);
-
-our @EXPORT_OK   = ( @EXPAND, @ARGCOUNT );
-our %EXPORT_TAGS = (
+@EXPORT_OK   = (@EXPAND, @ARGCOUNT);
+%EXPORT_TAGS = (
     expand   => [ @EXPAND   ],
     argcount => [ @ARGCOUNT ],
 );
-our $AUTOLOAD;
 
-require AppConfig::State;
 
 #------------------------------------------------------------------------
 # new(\%config, @vars)
@@ -68,9 +63,16 @@ require AppConfig::State;
 
 sub new {
     my $class = shift;
-    bless {
-        STATE => AppConfig::State->new(@_)
-    }, $class;
+
+    require AppConfig::State;
+
+    my $self = {
+	STATE => AppConfig::State->new(@_)
+    };
+
+    bless $self, $class;
+	
+    return $self;
 }
 
 
@@ -165,6 +167,7 @@ sub cgi {
     $cgi->parse(shift);
 }
 
+    
 #------------------------------------------------------------------------
 # AUTOLOAD
 #
@@ -173,7 +176,6 @@ sub cgi {
 # AppConfig::State object.
 #
 #------------------------------------------------------------------------
-
 sub AUTOLOAD {
     my $self = shift;
     my $method;
@@ -188,6 +190,7 @@ sub AUTOLOAD {
     $self->{ STATE }->$method(@_);
 }
 
+
 1;
 
 __END__
@@ -201,37 +204,34 @@ AppConfig - Perl5 module for reading configuration files and parsing command lin
     use AppConfig;
 
     # create a new AppConfig object
-    my $config = AppConfig->new( \%cfg );
+    my $config = AppConfig->new(\%cfg);
 
     # define a new variable
-    $config->define( $varname => \%varopts );
+    $config->define($varname => \%varopts);
 
     # create/define combined
-    my $config = AppConfig->new( \%cfg, 
-        $varname => \%varopts,
-        $varname => \%varopts,
-        ...
-    );
+    my $config = AppConfig->new(\%cfg, 
+	    $varname => \%varopts, $varname => \%varopts, ...);
 
     # set/get the value
-    $config->set( $varname, $value );
+    $config->set($varname, $value);
     $config->get($varname);
 
     # shortcut form
     $config->varname($value);
-    $config->varname;
+    $config->varname();
 
     # read configuration file
     $config->file($file);
 
     # parse command line options
-    $config->args(\@args);      # default to \@ARGV
+    $config->args(\@args);	# default to \@ARGV
 
     # advanced command line options with Getopt::Long
-    $config->getopt(\@args);    # default to \@ARGV
+    $config->getopt(\@args);	# default to \@ARGV
 
     # parse CGI parameters (GET method)
-    $config->cgi($query);       # default to $ENV{ QUERY_STRING }
+    $config->cgi($query);	# default to $ENV{ QUERY_STRING }
 
 =head1 OVERVIEW
 
@@ -289,19 +289,6 @@ Configuration files may be arranged in blocks as per the style of Win32
     header = $lib/header
     footer = $lib/footer
 
-You can also use Perl's "heredoc" syntax to define a large block of
-text in a configuration file.
-
-    multiline = <<FOOBAR
-    line 1
-    line 2
-    FOOBAR
-
-    paths  exe  = "${PATH}:${HOME}/.bin"
-    paths  link = <<'FOO'
-    ${LD_LIBARRAY_PATH}:${HOME}/lib
-    FOO
-
 Variables may also be set by parsing command line arguments.
 
     myapp -verbose -site kfs -file f1 -file f2
@@ -319,9 +306,9 @@ CGI script via the URL (GET method).
 
 AppConfig requires Perl 5.005 or later.  
 
-The L<Getopt::Long> and L<Test::More> modules should be installed.
-If you are using a recent version of Perl (e.g. 5.8.0) then these
-should already be installed.
+The Getopt::Long and Test::More modules should be installed.  If you
+are using a recent version of Perl (e.g. 5.8.0) then these should 
+already be installed.
 
 =head1 OBTAINING AND INSTALLING THE AppConfig MODULE BUNDLE
 
@@ -387,7 +374,7 @@ AppConfig modules.
 
 =head2 USING THE AppConfig MODULE
 
-To import and use the L<AppConfig> module the following line should 
+To import and use the AppConfig module the following line should 
 appear in your Perl script:
 
      use AppConfig;
@@ -404,7 +391,7 @@ AppConfig is implemented using object-oriented methods.  A
 new AppConfig object is created and initialised using the 
 new() method.  This returns a reference to a new AppConfig 
 object.
-
+       
     my $config = AppConfig->new();
 
 This will create and return a reference to a new AppConfig object.
@@ -417,14 +404,14 @@ to the AppConfig::State constructor.
 The first (optional) parameter may be a reference to a hash array
 containing configuration information.  
 
-    my $config = AppConfig->new( {
-            CASE   => 1,
-            ERROR  => \&my_error,
-            GLOBAL => { 
-                    DEFAULT  => "<unset>", 
-                    ARGCOUNT => ARGCOUNT_ONE,
-                },
-        } );
+    my $config = AppConfig->new({
+	    CASE   => 1,
+	    ERROR  => \&my_error,
+	    GLOBAL => { 
+		    DEFAULT  => "<unset>", 
+		    ARGCOUNT => ARGCOUNT_ONE,
+		},
+	});
 
 See L<AppConfig::State> for full details of the configuration options
 available.  These are, in brief:
@@ -461,7 +448,7 @@ Subsequent parameters may be variable definitions.  These are passed
 to the define() method, described below in L<DEFINING VARIABLES>.
 
     my $config = AppConfig->new("foo", "bar", "baz");
-    my $config = AppConfig->new( { CASE => 1 }, qw(foo bar baz) );
+    my $config = AppConfig->new({ CASE => 1 }, qw(foo bar baz));
 
 Note that any unresolved method calls to AppConfig are automatically 
 delegated to the AppConfig::State object.  In practice, it means that
@@ -494,20 +481,20 @@ reference to a hash array may also be passed to specify configuration
 information for the variable:
 
     $config->define("foo", {
-            DEFAULT   => 99,
-            ALIAS     => 'metavar1',
-        });
+	    DEFAULT   => 99,
+	    ALIAS     => 'metavar1',
+	});
 
 Configuration items specified in the GLOBAL option to the module 
 constructor are applied by default when variables are created.  e.g.
 
-    my $config = AppConfig->new( { 
-        GLOBAL => {
-            DEFAULT  => "<undef>",
-            ARGCOUNT => ARGCOUNT_ONE,
-        }
-    } );
-
+    my $config = AppConfig->new({ 
+	    GLOBAL => {
+		    DEFAULT  => "<undef>",
+		    ARGCOUNT => ARGCOUNT_ONE,
+		}
+	});
+    
     $config->define("foo");
     $config->define("bar", { ARGCOUNT => ARGCOUNT_NONE } );
 
@@ -515,15 +502,15 @@ is equivalent to:
 
     my $config = AppConfig->new();
 
-    $config->define( "foo", {
-        DEFAULT  => "<undef>",
-        ARGCOUNT => ARGCOUNT_ONE,
-    } );
+    $config->define("foo", {
+		DEFAULT  => "<undef>",
+		ARGCOUNT => ARGCOUNT_ONE,
+	    });
 
-    $config->define( "bar", 
-        DEFAULT  => "<undef>",
-        ARGCOUNT => ARGCOUNT_NONE,
-    } );
+    $config->define("bar", 
+		DEFAULT  => "<undef>",
+		ARGCOUNT => ARGCOUNT_NONE,
+	    });
 
 Multiple variables may be defined in the same call to define().
 Configuration hashes for variables can be omitted.
@@ -562,13 +549,11 @@ ARGCOUNT_LIST, C</%/> = ARGCOUNT_HASH, C</[=:].*/> = ARGCOUNT_ONE)
 =item EXPAND
 
 Specifies which variable expansion policies should be used when parsing 
-configuration files.  Constants in C<:expand> tag set define:
-
-    EXPAND_NONE - no expansion (default) 
-    EXPAND_VAR  - expand C<$var> or C<$(var)> as other variables
-    EXPAND_UID  - expand C<~> and C<~uid> as user's home directory 
-    EXPAND_ENV - expand C<${var}> as environment variable
-    EXPAND_ALL - do all expansions. 
+configuration files.  Constants in C<:expand> tag set define EXPAND_NONE
+- no expansion (default), EXPAND_VAR - expand C<$var> or C<$(var)> as 
+other AppConfig variables, EXPAND_UID - expand C<~uid> as user's home 
+directory, EXPAND_ENV - expand C<${var}> as environment variable,
+EXPAND_ALL - do all expansions.  May be logically or'd.
 
 =item VALIDATE
 
@@ -610,33 +595,34 @@ equivalent full specifications:
     $config->define("foo|bar|baz!");
 
     $config->define(
-            "foo" => { 
-                ALIAS    => "bar|baz", 
-                ARGCOUNT => ARGCOUNT_NONE,
-            });
+	    "foo" => { 
+		ALIAS    => "bar|baz", 
+		ARGCOUNT => ARGCOUNT_NONE,
+	    });
 
     $config->define("name=s");
 
     $config->define(
-            "name" => { 
-                ARGCOUNT => ARGCOUNT_ONE,
-            });
+	    "name" => { 
+		ARGCOUNT => ARGCOUNT_ONE,
+	    });
 
     $config->define("file|filelist|f=s@");
 
     $config->define(
-            "file" => { 
-                ALIAS    => "filelist|f", 
-                ARGCOUNT => ARGCOUNT_LIST,
-            });
+	    "file" => { 
+		ALIAS    => "filelist|f", 
+		ARGCOUNT => ARGCOUNT_LIST,
+	    });
+
 
     $config->define("user|u=s%");
 
     $config->define(
-            "user" => { 
-                ALIAS    => "u", 
-                ARGCOUNT => ARGCOUNT_HASH,
-            });
+	    "user" => { 
+		ALIAS    => "u", 
+		ARGCOUNT => ARGCOUNT_HASH,
+	    });
 
 Additional configuration options may be specified by hash reference, as per 
 normal.  The compact definition format will override any configuration 
@@ -745,8 +731,8 @@ a '\'.
     # this is a comment
     callsign = alpha bravo camel delta echo foxtrot golf hipowls \
                india juliet kilo llama mike november oscar papa  \
-               quebec romeo sierra tango umbrella victor whiskey \
-               x-ray yankee zebra
+	       quebec romeo sierra tango umbrella victor whiskey \
+	       x-ray yankee zebra
 
 Variables that are simple flags and do not expect an argument (ARGCOUNT = 
 ARGCOUNT_NONE) can be specified without any value.  They will be set with 
@@ -797,7 +783,7 @@ A reference to the hash is returned when the variable is requested.
 
     my $aliases = $config->alias();
     foreach my $k (keys %$aliases) {
-        print "$k => $aliases->{ $k }\n";
+	print "$k => $aliases->{ $k }\n";
     }
 
 The '-' prefix can be used to reset a variable to its default value and
@@ -893,7 +879,7 @@ is encountered.
                                          # $config->file('/tmp/bar')
 
     # file => [ '/tmp/foo', '/tmp/bar' ]
-
+    
     myprog -door "jim=Jim Morrison" -door "ray=Ray Manzarek"
                                     # $config->door("jim=Jim Morrison");
                                     # $config->door("ray=Ray Manzarek");
@@ -925,8 +911,8 @@ specification string is constructed from the name, any aliases (delimited
 by a vertical bar '|') and the value of the ARGS parameter.
 
     $config->define("foo", {
-        ARGS  => "=i",
-        ALIAS => "bar|baz",
+	ARGS  => "=i",
+	ALIAS => "bar|baz",
     });
 
     # Getopt::Long specification: "foo|bar|baz=i"
@@ -953,9 +939,9 @@ equals sign:
 
     http://www.here.com/cgi-bin/myscript?foo=bar&baz=qux&verbose
 
-        # $config->foo('bar');
-        # $config->baz('qux');
-        # $config->verbose(1);
+	# $config->foo('bar');
+	# $config->baz('qux');
+	# $config->verbose(1);
 
 Certain values specified in a URL must be escaped in the appropriate 
 manner (see CGI specifications at http://www.w3c.org/ for full details).  
@@ -964,13 +950,13 @@ to restore the parameters to their intended values.
 
     http://where.com/mycgi?title=%22The+Wrong+Trousers%22
 
-    # $config->title('"The Wrong Trousers"');
+	# $config->title('"The Wrong Trousers"');
 
 Please be considerate of the security implications of providing writeable
 access to script variables via CGI.
 
     http://rebel.alliance.com/cgi-bin/...
-        .../send_report?file=%2Fetc%2Fpasswd&email=darth%40empire.com
+	.../send_report?file=%2Fetc%2Fpasswd&email=darth%40empire.com
 
 To avoid any accidental or malicious changing of "private" variables, 
 define only the "public" variables before calling the cgi() (or any 
@@ -978,10 +964,10 @@ other) method.  Further variables can subequently be defined which
 can not be influenced by the CGI parameters.
 
     $config->define('verbose', 'debug')
-    $config->cgi();             # can only set verbose and debug
+    $config->cgi();		# can only set verbose and debug
 
     $config->define('email', 'file');
-    $config->file($cfgfile);    # can set verbose, debug, email + file
+    $config->file($cfgfile);	# can set verbose, debug, email + file
 
 
 =head1 CONSTANT DEFINITIONS
@@ -1030,13 +1016,13 @@ See AppConfig::State for full details of the use of these constants.
 
 Andy Wardley, E<lt>abw@wardley.orgE<gt>
 
-With contributions from Dave Viner, Ijon Tichy, Axel Gerstmair and
-many others whose names have been lost to the sands of time (reminders
-welcome).
+=head1 REVISION
+
+Revision $Revision: 1.4 $ distributed as part of AppConfig 1.54.
 
 =head1 COPYRIGHT
 
-Copyright (C) 1997-2007 Andy Wardley.  All Rights Reserved.
+Copyright (C) 1997-2003 Andy Wardley.  All Rights Reserved.
 
 Copyright (C) 1997,1998 Canon Research Centre Europe Ltd.
 
