@@ -10,13 +10,18 @@
 # Copyright (C) 1997-2003 Andy Wardley.  All Rights Reserved.
 # Copyright (C) 1997,1998 Canon Research Centre Europe Ltd.
 #
+# $Id: CGI.pm,v 1.2 2003/04/29 09:22:33 abw Exp $
+#
 #============================================================================
 
 package AppConfig::CGI;
-use strict;
-use warnings;
+
+require 5.004;
 use AppConfig::State;
-our $VERSION = '1.66_03';
+use strict;
+use vars qw( $VERSION );
+
+$VERSION = sprintf("%d.%02d", q$Revision: 1.2 $ =~ /(\d+)\.(\d+)/);
 
 
 #------------------------------------------------------------------------
@@ -34,16 +39,19 @@ our $VERSION = '1.66_03';
 sub new {
     my $class = shift;
     my $state = shift;
-    my $self  = {
-        STATE    => $state,                # AppConfig::State ref
-        DEBUG    => $state->_debug(),      # store local copy of debug
-        PEDANTIC => $state->_pedantic,     # and pedantic flags
-    };
-    bless $self, $class;
 
+    
+    my $self = {
+        STATE    => $state,                # AppConfig::State ref
+	DEBUG    => $state->_debug(),      # store local copy of debug
+	PEDANTIC => $state->_pedantic,     # and pedantic flags
+   };
+
+    bless $self, $class;
+	
     # call parse(@_) to parse any arg list passed 
     $self->parse(@_)
-        if @_;
+	if @_;
 
     return $self;
 }
@@ -66,14 +74,14 @@ sub parse {
     my $query    = shift;
     my $warnings = 0;
     my ($variable, $value, $nargs);
-
+    
 
     # take a local copy of the state to avoid much hash dereferencing
     my ($state, $debug, $pedantic) = @$self{ qw( STATE DEBUG PEDANTIC ) };
 
     # get the cgi query if not defined
     $query = $ENV{ QUERY_STRING }
-        unless defined $query;
+	unless defined $query;
 
     # no query to process
     return 1 unless defined $query;
@@ -86,42 +94,42 @@ sub parse {
 
     # install a closure as a new error handler
     $state->_ehandler(
-        sub {
-            # modify the error message 
-            my $format  = shift;
-            $format =~ s/</&lt;/g;
-            $format =~ s/>/&gt;/g;
-            $format  = "<p>\n<b>[ AppConfig::CGI error: </b>$format<b> ] </b>\n<p>\n";
-            # send error to stdout for delivery to web client
-            printf($format, @_);
-        }
+	sub {
+	    # modify the error message 
+	    my $format  = shift;
+	    $format =~ s/</&lt;/g;
+	    $format =~ s/>/&gt;/g;
+	    $format  = "<p>\n<b>[ AppConfig::CGI error: </b>$format<b> ] </b>\n<p>\n";
+	    # send error to stdout for delivery to web client
+	    printf($format, @_);
+	}
     );
 
 
     PARAM: foreach (split('&', $query)) {
 
-        # extract parameter and value from query token
-        ($variable, $value) = map { _unescape($_) } split('=');
+	# extract parameter and value from query token
+	($variable, $value) = map { _unescape($_) } split('=');
 
-        # check an argument was provided if one was expected
-        if ($nargs = $state->_argcount($variable)) {
-            unless (defined $value) {
-                $state->_error("$variable expects an argument");
-                $warnings++;
-                last PARAM if $pedantic;
-                next;
-            }
-        }
-        # default an undefined value to 1 if ARGCOUNT_NONE
-        else {
-            $value = 1 unless defined $value;
-        }
+	# check an argument was provided if one was expected
+	if ($nargs = $state->_argcount($variable)) {
+	    unless (defined $value) {
+		$state->_error("$variable expects an argument");
+		$warnings++;
+		last PARAM if $pedantic;
+		next;
+	    }
+	}
+	# default an undefined value to 1 if ARGCOUNT_NONE
+	else {
+	    $value = 1 unless defined $value;
+	}
 
-        # set the variable, noting any error
-        unless ($state->set($variable, $value)) {
-            $warnings++;
-            last PARAM if $pedantic;
-        }
+	# set the variable, noting any error
+	unless ($state->set($variable, $value)) {
+	    $warnings++;
+	    last PARAM if $pedantic;
+	}
     }
 
     # restore original error handler
@@ -215,9 +223,13 @@ module via the cgi() method.
 
 Andy Wardley, C<E<lt>abw@wardley.org<gt>>
 
+=head1 REVISION
+
+$Revision: 1.2 $
+
 =head1 COPYRIGHT
 
-Copyright (C) 1997-2007 Andy Wardley.  All Rights Reserved.
+Copyright (C) 1997-2003 Andy Wardley.  All Rights Reserved.
 
 Copyright (C) 1997,1998 Canon Research Centre Europe Ltd.
 
