@@ -7,17 +7,22 @@
 #
 # Written by Andy Wardley <abw@wardley.org>
 #
-# Copyright (C) 1997-2007 Andy Wardley.  All Rights Reserved.
+# Copyright (C) 1997-2003 Andy Wardley.  All Rights Reserved.
 # Copyright (C) 1997,1998 Canon Research Centre Europe Ltd.
 #
+# $Id: File.pm,v 1.6 2003/04/29 10:08:35 abw Exp $
+#
 #============================================================================
 
 package AppConfig::File;
-use strict;
-use warnings;
+
+require 5.005;
 use AppConfig;
 use AppConfig::State;
-our $VERSION = '1.66_03';
+use strict;
+use vars qw( $VERSION );
+
+$VERSION = sprintf("%d.%02d", q$Revision: 1.6 $ =~ /(\d+)\.(\d+)/);
 
 
 #------------------------------------------------------------------------
@@ -34,16 +39,18 @@ our $VERSION = '1.66_03';
 sub new {
     my $class = shift;
     my $state = shift;
-    my $self  = {
-        STATE    => $state,                # AppConfig::State ref
-        DEBUG    => $state->_debug(),      # store local copy of debug 
-        PEDANTIC => $state->_pedantic,     # and pedantic flags
+    
+    my $self = {
+	STATE    => $state,                # AppConfig::State ref
+	DEBUG    => $state->_debug(),      # store local copy of debug 
+	PEDANTIC => $state->_pedantic,     # and pedantic flags
     };
 
     bless $self, $class;
 
     # call parse(@_) to parse any files specified as further params
-    $self->parse(@_) if @_;
+    $self->parse(@_)
+	if @_;
 
     return $self;
 }
@@ -90,67 +97,67 @@ sub parse {
 
     # ...and if it doesn't exist, we craft a default handler
     $errhandler = sub { warn(sprintf(shift, @_), "\n") }
-        unless defined $errhandler;
+	unless defined $errhandler;
 
     # install a closure as a new error handler
     $state->_ehandler(
-        sub {
-            # modify the error message 
-            my $format  = shift;
-               $format .= ref $file 
-                          ? " at line $."
-                          : " at $file line $.";
+	sub {
+	    # modify the error message 
+	    my $format  = shift;
+	       $format .= ref $file 
+	                  ? " at line $."
+	                  : " at $file line $.";
 
-            # chain call to prevous handler
-            &$errhandler($format, @_);
-        }
+	    # chain call to prevous handler
+	    &$errhandler($format, @_);
+	}
     );
 
     # trawl through all files passed as params
     FILE: while ($file = shift) {
 
-        # local/lexical vars ensure opened files get closed
-        my $handle;
-        local *FH;
+	# local/lexical vars ensure opened files get closed
+	my $handle;
+	local *FH;
 
-        # if the file is a reference, we assume it's a file handle, if
-        # not, we assume it's a filename and attempt to open it
-        $handle = $file;
-        if (ref($file)) {
-            $handle = $file;
+	# if the file is a reference, we assume it's a file handle, if
+	# not, we assume it's a filename and attempt to open it
+	$handle = $file;
+	if (ref($file)) {
+	    $handle = $file;
 
-            # DEBUG
-            print STDERR "reading from file handle: $file\n" if $debug;
-        }
-        else {
-            # open and read config file
-            open(FH, $file) or do {
-                # restore original error handler and report error
-                $state->_ehandler($errhandler);
-                $state->_error("$file: $!");
+	    # DEBUG
+	    print STDERR "reading from file handle: $file\n" if $debug;
+	}
+	else {
+	    # open and read config file
+	    open(FH, $file) or do {
+		# restore original error handler and report error
+	    	$state->_ehandler($errhandler);
+		$state->_error("$file: $!");
 
-                return undef;
-            };
-            $handle = \*FH;
+		return undef;
+	    };
+	    $handle = \*FH;
 
-            # DEBUG
-            print STDERR "reading file: $file\n" if $debug;
-        }
+	    # DEBUG
+	    print STDERR "reading file: $file\n" if $debug;
+	}
 
-        # initialise $prefix to nothing (no [block])
-        $prefix = '';
+	# initialise $prefix to nothing (no [block])
+	$prefix = '';
 
-        while (<$handle>) {
-            chomp;
+	while (<$handle>) {
+	    chomp;
 
             # Throw away everything from an unescaped # to EOL
             s/(^|\s+)#.*/$1/;
 
-            # add next line if there is one and this is a continuation
-            if (s/\\$// && !eof($handle)) {
-                $_ .= <$handle>;
-                redo;
-            }
+	    # add next line if there is one and this is a continuation
+	    if (s/\\$// && !eof($handle)) {
+		$_ .= <$handle>;
+		redo;
+	    }
 
             # Convert \# -> #
             s/\\#/#/g;
@@ -158,122 +165,109 @@ sub parse {
             # ignore blank lines
             next if /^\s*$/;
 
-            # strip leading and trailing whitespace
-            s/^\s+//;
-            s/\s+$//;
+	    # strip leading and trailing whitespace
+	    s/^\s+//;
+	    s/\s+$//;
 
-            # look for a [block] to set $prefix
-            if (/^\[([^\]]+)\]$/) {
-                $prefix = $1;
-                print STDERR "Entering [$prefix] block\n" if $debug;
-                next;
-            }
+	    # look for a [block] to set $prefix
+	    if (/^\[([^\]]+)\]$/) {
+		$prefix = $1;
+		print STDERR "Entering [$prefix] block\n" if $debug;
+		next;
+	    }
 
-            # split line up by whitespace (\s+) or "equals" (\s*=\s*)
-            if (/^([^\s=]+)(?:(?:(?:\s*=\s*)|\s+)(.*))?/) {
-                my ($variable, $value) = ($1, $2);
+	    # split line up by whitespace (\s+) or "equals" (\s*=\s*)
+	    if (/^([^\s=]+)(?:(?:(?:\s*=\s*)|\s+)(.*))?/) {
+		my ($variable, $value) = ($1, $2);
 
-                if (defined $value) {
-                    # here document
-                    if ($value =~ /^([^\s=]+\s*=)?\s*<<(['"]?)(\S+)\2$/) { # '<<XX' or 'hashkey =<<XX'
-                        my $boundary = "$3\n";
-                        $value = defined($1) ? $1 : '';
-                        while (<$handle>) {
-                            last if $_ eq $boundary;
-                            $value .= $_;
-                        };
-                        $value =~ s/[\r\n]$//;
-                    } else {
-                        # strip any quoting from the variable value
-                        $value =~ s/^(['"])(.*)\1$/$2/;
-                    };
-                };
+		# strip any quoting from the variable value
+		$value =~ s/^(['"])(.*)\1$/$2/ if defined $value;
 
-                # strip any leading '+/-' from the variable
-                $variable =~ s/^([\-+]?)//;
-                $flag = $1;
+		# strip any leading '+/-' from the variable
+		$variable =~ s/^([\-+]?)//;
+		$flag = $1;
 
-                # $variable gets any $prefix 
-                $variable = $prefix . '_' . $variable
-                    if length $prefix;
+		# $variable gets any $prefix 
+		$variable = $prefix . '_' . $variable
+		    if length $prefix;
 
-                # if the variable doesn't exist, we call set() to give 
-                # AppConfig::State a chance to auto-create it
-                unless ($state->_exists($variable) 
-                            || $state->set($variable, 1)) {
-                    $warnings++;
-                    last FILE if $pedantic;
-                    next;
-                }       
+		# if the variable doesn't exist, we call set() to give 
+		# AppConfig::State a chance to auto-create it
+		unless ($state->_exists($variable) 
+			    || $state->set($variable, 1)) {
+		    $warnings++;
+		    last FILE if $pedantic;
+		    next;
+		}	
 
-                my $nargs = $state->_argcount($variable);
+		my $nargs = $state->_argcount($variable);
 
-                # variables prefixed '-' are reset to their default values
-                if ($flag eq '-') {
-                    $state->_default($variable);
-                    next;
-                }
-                # those prefixed '+' get set to 1
-                elsif ($flag eq '+') {
-                    $value = 1 unless defined $value;
-                }
+		# variables prefixed '-' are reset to their default values
+		if ($flag eq '-') {
+		    $state->_default($variable);
+		    next;
+		}
+		# those prefixed '+' get set to 1
+		elsif ($flag eq '+') {
+		    $value = 1 unless defined $value;
+		}
 
-                # determine if any extra arguments were expected
-                if ($nargs) {
-                    if (defined $value && length $value) {
-                        # expand any embedded variables, ~uids or
-                        # environment variables, testing the return value
-                        # for errors;  we pass in any variable-specific
-                        # EXPAND value 
-                        unless ($self->_expand(\$value, 
-                                $state->_expand($variable), $prefix)) {
-                            print STDERR "expansion of [$value] failed\n" 
-                                if $debug;
-                            $warnings++;
-                            last FILE if $pedantic;
-                        }
-                    }
-                    else {
-                        $state->_error("$variable expects an argument");
-                        $warnings++;
-                        last FILE if $pedantic;
-                        next;
-                    }
-                }
-                # $nargs = 0
-                else {
-                    # default value to 1 unless it is explicitly defined
-                    # as '0' or "off"
-                    if (defined $value) {
-                        # "off" => 0
-                        $value = 0 if $value =~ /off/i;
-                        # any value => 1
-                        $value = 1 if $value;
-                    }
-                    else {
-                        # assume 1 unless explicitly defined off/0
-                        $value = 1;
-                    }
-                    print STDERR "$variable => $value (no expansion)\n"
-                        if $debug;
-                }
-
-                # set the variable, noting any failure from set()
-                unless ($state->set($variable, $value)) {
-                    $warnings++;
-                    last FILE if $pedantic;
-                }
-            }
-            else {
-                $state->_error("parse error");
-                $warnings++;
-            }
-        }
+		# determine if any extra arguments were expected
+		if ($nargs) {
+		    if (defined $value && length $value) {
+			# expand any embedded variables, ~uids or
+			# environment variables, testing the return value
+			# for errors;  we pass in any variable-specific
+			# EXPAND value 
+			unless ($self->_expand(\$value, 
+				$state->_expand($variable), $prefix)) {
+			    print STDERR "expansion of [$value] failed\n" 
+				if $debug;
+			    $warnings++;
+			    last FILE if $pedantic;
+			}
+		    }
+		    else {
+			$state->_error("$variable expects an argument");
+			$warnings++;
+			last FILE if $pedantic;
+			next;
+		    }
+		}
+		# $nargs = 0
+		else {
+		    # default value to 1 unless it is explicitly defined
+		    # as '0' or "off"
+		    if (defined $value) {
+			# "off" => 0
+    			$value = 0 if $value =~ /off/i;
+			# any value => 1
+			$value = 1 if $value;
+		    }
+		    else {
+			# assume 1 unless explicitly defined off/0
+    			$value = 1;
+		    }
+		    print STDERR "$variable => $value (no expansion)\n"
+			if $debug;
+		}
+	   
+		# set the variable, noting any failure from set()
+		unless ($state->set($variable, $value)) {
+		    $warnings++;
+		    last FILE if $pedantic;
+		}
+	    }
+	    else {
+		$state->_error("parse error");
+		$warnings++;
+	    }
+	}
     }
 
     # restore original error handler
     $state->_ehandler($errhandler);
-
+    
     # return $warnings => 0, $success => 1
     return $warnings ? 0 : 1;
 }
@@ -323,8 +317,8 @@ sub _expand {
     # create an AppConfig::Sys instance, or re-use a previous one, 
     # to handle platform dependant functions: getpwnam(), getpwuid()
     unless ($sys = $self->{ SYS }) {
-        require AppConfig::Sys;
-        $sys = $self->{ SYS } = AppConfig::Sys->new();
+	require AppConfig::Sys;
+	$sys = $self->{ SYS } = AppConfig::Sys->new();
     }
 
     print STDERR "Expansion of [$$value] " if $debug;
@@ -332,129 +326,138 @@ sub _expand {
     EXPAND: {
 
         # 
-        # EXPAND_VAR
-        # expand $(var) and $var as AppConfig::State variables
-        #
-        if ($expand & AppConfig::EXPAND_VAR) {
+	# EXPAND_VAR
+	# expand $(var) and $var as AppConfig::State variables
+	#
+	if ($expand & AppConfig::EXPAND_VAR) {
 
-            $$value =~ s{
+	    $$value =~ s{
                 (?<!\\)\$ (?: \((\w+)\) | (\w+) ) # $2 => $(var) | $3 => $var
 
-            } {
-                # embedded variable name will be one of $2 or $3
-                $var = defined $1 ? $1 : $2;
+	    } {
+		# embedded variable name will be one of $2 or $3
+		$var = defined $1 ? $1 : $2;
 
-                # expand the variable if defined
-                if ($state->_exists($var)) {
-                    $val = $state->get($var);
-                }
-                elsif (length $prefix 
-                        && $state->_exists($prefix . '_' . $var)) {
-                    print STDERR "(\$$var => \$${prefix}_$var) "
-                        if $debug;
-                    $var = $prefix . '_' . $var;
-                    $val = $state->get($var);
-                }
-                else {
-                    # raise a warning if EXPAND_WARN set
-                    if ($expand & AppConfig::EXPAND_WARN) {
-                        $state->_error("$var: no such variable");
-                        $warnings++;
-                    }
+		# expand the variable if defined
+		if ($state->_exists($var)) {
+		    $val = $state->get($var);
+		}
+		elsif (length $prefix 
+			&& $state->_exists($prefix . '_' . $var)) {
+		    print STDERR "(\$$var => \$${prefix}_$var) "
+			if $debug;
+		    $var = $prefix . '_' . $var;
+		    $val = $state->get($var);
+		}
+		else {
+		    # raise a warning if EXPAND_WARN set
+		    if ($expand & AppConfig::EXPAND_WARN) {
+			$state->_error("$var: no such variable");
+			$warnings++;
+		    }
 
-                    # replace variable with nothing
-                    $val = '';
-                }
+		    # replace variable with nothing
+		    $val = '';
+		}
 
-                # $val gets substituted back into the $value string
-                $val;
-            }gex;
+		# $val gets substituted back into the $value string
+		$val;
+	    }gex;
 
             $$value =~ s/\\\$/\$/g;
 
-            # bail out now if we need to
-            last EXPAND if $warnings && $pedantic;
-        }
+	    # bail out now if we need to
+	    last EXPAND if $warnings && $pedantic;
+	}
 
 
-        #
-        # EXPAND_UID
-        # expand ~uid as home directory (for $< if uid not specified)
-        #
-        if ($expand & AppConfig::EXPAND_UID) {
-            $$value =~ s{
-                ~(\w+)?                    # $1 => username (optional)
-            } {
-                $val = undef;
+	#
+	# EXPAND_UID
+	# expand ~uid as home directory (for $< if uid not specified)
+	#
+	if ($expand & AppConfig::EXPAND_UID) {
 
-                # embedded user name may be in $1
-                if (defined ($var = $1)) {
-                    # try and get user's home directory
-                    if ($sys->can_getpwnam()) {
-                        $val = ($sys->getpwnam($var))[7];
-                    }
-                } else {
-                    # determine home directory 
-                    $val = $ENV{ HOME };
-                }
+	    $$value =~ s{
+		~(\w+)?                    # $1 => username (optional)
+	    } {
+		$val = undef;
 
-                # catch-all for undefined $dir
-                unless (defined $val) {
-                    # raise a warning if EXPAND_WARN set
-                    if ($expand & AppConfig::EXPAND_WARN) {
-                        $state->_error("cannot determine home directory%s",
-                            defined $var ? " for $var" : "");
-                        $warnings++;
-                    }
+		# embedded user name may be in $1
+		if (defined ($var = $1)) {
+		    # try and get user's home directory
+		    if ($sys->can_getpwnam()) {
+			$val = ($sys->getpwnam($var))[7];
+		    }
+		}
+		else {
+		    # determine home directory 
+		    unless (defined($val = $self->{ HOME })) {
+			if (exists $ENV{ HOME }) {
+			    $val = $ENV{ HOME };
+			}
+			elsif ($sys->can_getpwuid()) {
+			    $val = ($sys->getpwuid($<))[7];
+			}
 
-                    # replace variable with nothing
-                    $val = '';
-                }
+			# cache value for next time
+			$self->{ HOME } = $val;
+		    }
+		}
 
-                # $val gets substituted back into the $value string
-                $val;
-            }gex;
+		# catch-all for undefined $dir
+		unless (defined $val) {
+		    # raise a warning if EXPAND_WARN set
+		    if ($expand & AppConfig::EXPAND_WARN) {
+			$state->_error("cannot determine home directory%s",
+			    defined $var ? " for $var" : "");
+			$warnings++;
+		    }
 
-            # bail out now if we need to
-            last EXPAND if $warnings && $pedantic;
-        }
+		    # replace variable with nothing
+		    $val = '';
+		}
+
+		# $val gets substituted back into the $value string
+		$val;
+	    }gex;
+
+	    # bail out now if we need to
+	    last EXPAND if $warnings && $pedantic;
+	}
 
 
-        #
-        # EXPAND_ENV
-        # expand ${VAR} as environment variables
-        #
-        if ($expand & AppConfig::EXPAND_ENV) {
+	#
+	# EXPAND_ENV
+	# expand ${VAR} as environment variables
+	#
+	if ($expand & AppConfig::EXPAND_ENV) {
 
-            $$value =~ s{ 
-                ( \$ \{ (\w+) \} )
-            } {
-                $var = $2;
+	    $$value =~ s{ 
+		( \$ \{ (\w+) \} )
+	    } {
+		$var = $2;
 
-                # expand the variable if defined
-                if (exists $ENV{ $var }) {
-                    $val = $ENV{ $var };
-                } elsif ( $var eq 'HOME' ) {
-                    # In the special case of HOME, if not set
-                    # use the internal version
-                    $val = $self->{ HOME };
-                } else {
-                    # raise a warning if EXPAND_WARN set
-                    if ($expand & AppConfig::EXPAND_WARN) {
-                        $state->_error("$var: no such environment variable");
-                        $warnings++;
-                    }
+		# expand the variable if defined
+		if (exists $ENV{ $var }) {
+		    $val = $ENV{ $var };
+		}
+		else {
+		    # raise a warning if EXPAND_WARN set
+		    if ($expand & AppConfig::EXPAND_WARN) {
+			$state->_error("$var: no such environment variable");
+			$warnings++;
+		    }
 
-                    # replace variable with nothing
-                    $val = '';
-                }
-                # $val gets substituted back into the $value string
-                $val;
-            }gex;
+		    # replace variable with nothing
+		    $val = '';
+		}
+		# $val gets substituted back into the $value string
+		$val;
+	    }gex;
 
-            # bail out now if we need to
-            last EXPAND if $warnings && $pedantic;
-        }
+	    # bail out now if we need to
+	    last EXPAND if $warnings && $pedantic;
+	}
     }
 
     print STDERR "=> [$$value] (EXPAND = $expand)\n" if $debug;
@@ -475,9 +478,9 @@ sub _dump {
     my $self = shift;
 
     foreach my $key (keys %$self) {
-        printf("%-10s => %s\n", $key, 
-                defined($self->{ $key }) ? $self->{ $key } : "<undef>");
-    }       
+	printf("%-10s => %s\n", $key, 
+		defined($self->{ $key }) ? $self->{ $key } : "<undef>");
+    }	    
 } 
 
 
@@ -524,7 +527,7 @@ AppConfig::File object is created and initialised using the
 AppConfig::File->new() method.  This returns a reference to a new 
 AppConfig::File object.  A reference to an AppConfig::State object 
 should be passed in as the first parameter:
-
+       
     my $state   = AppConfig::State->new();
     my $cfgfile = AppConfig::File->new($state);
 
@@ -575,8 +578,8 @@ line with a '\'.
 
     callsign = alpha bravo camel delta echo foxtrot golf hipowls \
                india juliet kilo llama mike november oscar papa  \
-               quebec romeo sierra tango umbrella victor whiskey \
-               x-ray yankee zebra
+	       quebec romeo sierra tango umbrella victor whiskey \
+	       x-ray yankee zebra
 
 Variables that are simple flags and do not expect an argument (ARGCOUNT = 
 ARGCOUNT_NONE) can be specified without any value.  They will be set with 
@@ -626,33 +629,9 @@ A reference to the hash is returned when the variable is requested.
 
     my $aliases = $config->alias();
     foreach my $k (keys %$aliases) {
-        print "$k => $aliases->{ $k }\n";
+	print "$k => $aliases->{ $k }\n";
     }
 
-A large chunk of text can be defined using Perl's "heredoc" quoting
-style.
-
-   scalar = <<BOUNDARY_STRING
-   line 1
-   line 2: Space/linebreaks within a HERE document are kept.
-   line 3: The last linebreak (\n) is stripped.
-   BOUNDARY_STRING
-
-   hash   key1 = <<'FOO'
-     * Quotes (['"]) around the boundary string are simply ignored.
-     * Whether the variables in HERE document are expanded depends on
-       the EXPAND option of the variable or global setting.
-   FOO
-
-   hash = key2 = <<"_bar_"
-   Text within HERE document are kept as is.
-   # comments are treated as a normal text.
-   The same applies to line continuation. \
-   _bar_
-
-Note that you cannot use HERE document as a key in a hash or a name 
-of a variable.
-
 The '-' prefix can be used to reset a variable to its default value and
 the '+' prefix can be used to set it to 1
 
@@ -694,9 +673,15 @@ file.
 
 Andy Wardley, E<lt>abw@wardley.orgE<gt>
 
+=head1 REVISION
+
+$Revision: 1.6 $
+
 =head1 COPYRIGHT
 
-Copyright (C) 1997-2007 Andy Wardley.  All Rights Reserved.
+Copyright (C) 1997-2003 Andy Wardley.  All Rights Reserved.
+
+Copyright (C) 1997,1998 Canon Research Centre Europe Ltd.
 
 This module is free software; you can redistribute it and/or modify it 
 under the same terms as Perl itself.
