@@ -9,9 +9,11 @@
 #
 # Written by Andy Wardley <abw@wardley.org>
 #
-# Copyright (C) 1997-2007 Andy Wardley.  All Rights Reserved.
+# Copyright (C) 1997-2003 Andy Wardley.  All Rights Reserved.
 # Copyright (C) 1997,1998 Canon Research Centre Europe Ltd.
 #
+# $Id: State.pm,v 1.2 2003/04/29 09:22:33 abw Exp $
+#
 #----------------------------------------------------------------------------
 #
 # TODO
@@ -37,15 +39,17 @@
 #============================================================================
 
 package AppConfig::State;
-use strict;
-use warnings;
 
-our $VERSION = '1.66_03';
-our $DEBUG   = 0;
-our $AUTOLOAD;
+require 5.004;
+
+use strict;
+use vars qw( $VERSION $AUTOLOAD $DEBUG );
 
 # need access to AppConfig::ARGCOUNT_*
-use AppConfig ':argcount';
+use AppConfig qw(:argcount);
+
+$VERSION = sprintf("%d.%02d", q$Revision: 1.2 $ =~ /(\d+)\.(\d+)/);
+$DEBUG   = 0;
 
 # internal per-variable hashes that AUTOLOAD should provide access to
 my %METHVARS;
@@ -74,37 +78,37 @@ my @GLOBAL_OK = qw( DEFAULT EXPAND VALIDATE ACTION ARGS ARGCOUNT );
 
 sub new {
     my $class = shift;
-
+    
     my $self = {
-        # internal hash arrays to store variable specification information
-        VARIABLE   => { },     # variable values
-        DEFAULT    => { },     # default values
-        ALIAS      => { },     # known aliases  ALIAS => VARIABLE
-        ALIASES    => { },     # reverse alias lookup VARIABLE => ALIASES
-        ARGCOUNT   => { },     # arguments expected
-        ARGS       => { },     # specific argument pattern (AppConfig::Getopt)
-        EXPAND     => { },     # variable expansion (AppConfig::File)
-        VALIDATE   => { },     # validation regexen or functions
-        ACTION     => { },     # callback functions for when variable is set
-        GLOBAL     => { },     # default global settings for new variables
+	# internal hash arrays to store variable specification information
+	VARIABLE   => { },     # variable values
+	DEFAULT    => { },     # default values
+	ALIAS      => { },     # known aliases  ALIAS => VARIABLE
+	ALIASES    => { },     # reverse alias lookup VARIABLE => ALIASES
+	ARGCOUNT   => { },     # arguments expected
+	ARGS       => { },     # specific argument pattern (AppConfig::Getopt)
+	EXPAND     => { },     # variable expansion (AppConfig::File)
+	VALIDATE   => { },     # validation regexen or functions
+	ACTION     => { },     # callback functions for when variable is set
+	GLOBAL     => { },     # default global settings for new variables
 
-        # other internal data
-        CREATE     => 0,       # auto-create variables when set
-        CASE       => 0,       # case sensitivity flag (1 = sensitive)
-        PEDANTIC   => 0,       # return immediately on parse warnings
-        EHANDLER   => undef,   # error handler (let's hope we don't need it!)
-        ERROR      => '',      # error message
+	# other internal data
+	CREATE     => 0,       # auto-create variables when set
+	CASE       => 0,       # case sensitivity flag (1 = sensitive)
+	PEDANTIC   => 0,       # return immediately on parse warnings
+	EHANDLER   => undef,   # error handler (let's hope we don't need it!)
+	ERROR      => '',      # error message
     };
 
     bless $self, $class;
-
+	
     # configure if first param is a config hash ref
     $self->_configure(shift)
-        if ref($_[0]) eq 'HASH';
+	if ref($_[0]) eq 'HASH';
 
     # call define(@_) to handle any variables definitions
     $self->define(@_)
-        if @_;
+	if @_;
 
     return $self;
 }
@@ -133,108 +137,109 @@ sub define {
     my ($var, $args, $count, $opt, $val, $cfg, @names);
 
     while (@_) {
-        $var = shift;
-        $cfg = ref($_[0]) eq 'HASH' ? shift : { };
+	$var = shift;
+	$cfg = ref($_[0]) eq 'HASH' ? shift : { };
 
-        # variable may be specified in compact format, 'foo|bar=i@'
-        if ($var =~ s/(.+?)([!+=:].*)/$1/) {
+	# variable may be specified in compact format, 'foo|bar=i@'
+	if ($var =~ s/(.+?)([!+=:].*)/$1/) {
 
-            # anything coming after the name|alias list is the ARGS
-            $cfg->{ ARGS } = $2
-                if length $2;
-        }
+	    # anything coming after the name|alias list is the ARGS
+	    $cfg->{ ARGS } = $2
+		if length $2;
+	}
 
-        # examine any ARGS option
-        if (defined ($args = $cfg->{ ARGS })) {
-          ARGGCOUNT: {
-              $count = ARGCOUNT_NONE, last if $args =~ /^!/;
-              $count = ARGCOUNT_LIST, last if $args =~ /@/;
-              $count = ARGCOUNT_HASH, last if $args =~ /%/;
-              $count = ARGCOUNT_ONE;
-          }
-            $cfg->{ ARGCOUNT } = $count;
-        }
+	# examine any ARGS option
+	if (defined ($args = $cfg->{ ARGS })) {
+	    ARGGCOUNT: {
+		$count = ARGCOUNT_NONE, last if $args =~ /^!/;
+		$count = ARGCOUNT_LIST, last if $args =~ /@/;
+		$count = ARGCOUNT_HASH, last if $args =~ /%/;
+		$count = ARGCOUNT_ONE;
+	    }
 
-        # split aliases out
-        @names = split(/\|/, $var);
-        $var = shift @names;
-        $cfg->{ ALIAS } = [ @names ] if @names;
+	    $cfg->{ ARGCOUNT } = $count;
+	}
 
-        # variable name gets folded to lower unless CASE sensitive
-        $var = lc $var unless $self->{ CASE };
+	# split aliases out
+	@names = split(/\|/, $var);
+	$var = shift @names;
+	$cfg->{ ALIAS } = [ @names ] if @names;
 
-        # activate $variable (so it does 'exist()') 
-        $self->{ VARIABLE }->{ $var } = undef;
+    	# variable name gets folded to lower unless CASE sensitive
+	$var = lc $var unless $self->{ CASE };
 
-        # merge GLOBAL and variable-specific configurations
-        $cfg = { %{ $self->{ GLOBAL } }, %$cfg };
+	# activate $variable (so it does 'exist()') 
+	$self->{ VARIABLE }->{ $var } = undef;
 
-        # examine each variable configuration parameter
-        while (($opt, $val) = each %$cfg) {
-            $opt = uc $opt;
+	# merge GLOBAL and variable-specific configurations
+	$cfg = { %{ $self->{ GLOBAL } }, %$cfg };
 
-            # DEFAULT, VALIDATE, EXPAND, ARGS and ARGCOUNT are stored as 
-            # they are;
-            $opt =~ /^DEFAULT|VALIDATE|EXPAND|ARGS|ARGCOUNT$/ && do {
-                $self->{ $opt }->{ $var } = $val;
-                next;
-            };
+	# examine each variable configuration parameter
+	while (($opt, $val) = each %$cfg) {
+	    $opt = uc $opt;
 
-            # CMDARG has been deprecated
-            $opt eq 'CMDARG' && do {
-                $self->_error("CMDARG has been deprecated.  "
-                              . "Please use an ALIAS if required.");
-                next;
-            };
+	    # DEFAULT, VALIDATE, EXPAND, ARGS and ARGCOUNT are stored as 
+	    # they are;
+	    $opt =~ /^DEFAULT|VALIDATE|EXPAND|ARGS|ARGCOUNT$/ && do {
+		$self->{ $opt }->{ $var } = $val;
+		next;
+	    };
 
-            # ACTION should be a code ref
-            $opt eq 'ACTION' && do {
-                unless (ref($val) eq 'CODE') {
-                    $self->_error("'$opt' value is not a code reference");
-                    next;
-                };
+	    # CMDARG has been deprecated
+	    $opt eq 'CMDARG' && do {
+		$self->_error("CMDARG has been deprecated.  "
+			. "Please use an ALIAS if required.");
+		next;
+	    };
 
-                # store code ref, forcing keyword to upper case
-                $self->{ ACTION }->{ $var } = $val;
+	    # ACTION should be a code ref
+	    $opt eq 'ACTION' && do {
+		unless (ref($val) eq 'CODE') {
+		    $self->_error("'$opt' value is not a code reference");
+		    next;
+		};
 
-                next;
-            };
+		# store code ref, forcing keyword to upper case
+		$self->{ ACTION }->{ $var } = $val;
 
-            # ALIAS creates alias links to the variable name
-            $opt eq 'ALIAS' && do {
+		next;
+	    };
 
-                # coerce $val to an array if not already so
-                $val = [ split(/\|/, $val) ]
-                    unless ref($val) eq 'ARRAY';
+	    # ALIAS creates alias links to the variable name
+	    $opt eq 'ALIAS' && do {
 
-                # fold to lower case unless CASE sensitivity set
-                unless ($self->{ CASE }) {
-                    @$val = map { lc } @$val;
-                }
+		# coerce $val to an array if not already so
+		$val = [ split(/\|/, $val) ]
+		    unless ref($val) eq 'ARRAY';
+   
+		# fold to lower case unless CASE sensitivity set
+		unless ($self->{ CASE }) {
+		    @$val = map { lc } @$val;
+		}
 
-                # store list of aliases...
-                $self->{ ALIASES }->{ $var } = $val;
+		# store list of aliases...
+		$self->{ ALIASES }->{ $var } = $val;
 
-                # ...and create ALIAS => VARIABLE lookup hash entries
-                foreach my $a (@$val) {
-                    $self->{ ALIAS }->{ $a } = $var;
-                }
+		# ...and create ALIAS => VARIABLE lookup hash entries
+		foreach my $a (@$val) {
+		    $self->{ ALIAS }->{ $a } = $var;
+		}
 
-                next;
-            };
+		next;
+	    };
 
-            # default 
-            $self->_error("$opt is not a valid configuration item");
-        }
+	    # default 
+	    $self->_error("$opt is not a valid configuration item");
+	}
 
-        # set variable to default value
-        $self->_default($var);
+    	# set variable to default value
+	$self->_default($var);
 
-        # DEBUG: dump new variable definition
-        if ($DEBUG) {
-            print STDERR "Variable defined:\n";
-            $self->_dump_var($var);
-        }
+	# DEBUG: dump new variable definition
+	if ($DEBUG) {
+	    print STDERR "Variable defined:\n";
+    	    $self->_dump_var($var);
+	}
     }
 }
 
@@ -259,17 +264,17 @@ sub get {
 
     # check the variable has been defined
     unless (exists($self->{ VARIABLE }->{ $variable })) {
-        $self->_error("$variable: no such variable");
-        return undef;
+	$self->_error("$variable: no such variable");
+	return undef;
     }
 
     # DEBUG
     print STDERR "$self->get($variable) => ", 
-           defined $self->{ VARIABLE }->{ $variable }
-                  ? $self->{ VARIABLE }->{ $variable }
-                  : "<undef>",
-          "\n"
-          if $DEBUG;
+	   defined $self->{ VARIABLE }->{ $variable }
+		  ? $self->{ VARIABLE }->{ $variable }
+		  : "<undef>",
+	  "\n"
+	if $DEBUG;
 
     # return variable value, possibly negated if the name was "no<var>"
     $value = $self->{ VARIABLE }->{ $variable };
@@ -301,38 +306,38 @@ sub set {
 
     # check the variable exists
     if (exists($self->{ VARIABLE }->{ $variable })) {
-        # variable found, so apply any value negation
-        $value = $value ? 0 : 1 if $negate;
+	# variable found, so apply any value negation
+	$value = $value ? 0 : 1 if $negate;
     }
     else {
-        # auto-create variable if CREATE is 1 or a pattern matching 
-        # the variable name (real name, not an alias)
-        $create = $self->{ CREATE };
-        if (defined $create
-            && ($create eq '1' || $variable =~ /$create/)) {
-            $self->define($variable);
+	# auto-create variable if CREATE is 1 or a pattern matching 
+	# the variable name (real name, not an alias)
+	$create = $self->{ CREATE };
+	if (defined $create
+		&& ($create eq '1' || $variable =~ /$create/)) {
+	    $self->define($variable);
 
-            print STDERR "Auto-created $variable\n" if $DEBUG;
-        }
-        else {
-            $self->_error("$variable: no such variable");
-            return 0;
-        }
+	    print STDERR "Auto-created $variable\n" if $DEBUG;
+	}
+	else {
+	    $self->_error("$variable: no such variable");
+	    return 0;
+	}
     }
 
     # call the validate($variable, $value) method to perform any validation
     unless ($self->_validate($variable, $value)) {
-        $self->_error("$variable: invalid value: $value");
-        return 0;
+	$self->_error("$variable: invalid value: $value");
+	return 0;
     }
 
     # DEBUG
     print STDERR "$self->set($variable, ", 
-    defined $value
-        ? $value
-        : "<undef>",
-        ")\n"
-        if $DEBUG;
+	   defined $value
+		  ? $value
+		  : "<undef>",
+	  ")\n"
+	if $DEBUG;
 
 
     # set the variable value depending on its ARGCOUNT
@@ -340,25 +345,25 @@ sub set {
     $argcount = AppConfig::ARGCOUNT_ONE unless defined $argcount;
 
     if ($argcount eq AppConfig::ARGCOUNT_LIST) {
-        # push value onto the end of the list
-        push(@{ $self->{ VARIABLE }->{ $variable } }, $value);
+	# push value onto the end of the list
+	push(@{ $self->{ VARIABLE }->{ $variable } }, $value);
     }
     elsif ($argcount eq AppConfig::ARGCOUNT_HASH) {
-        # insert "<key>=<value>" data into hash 
-        my ($k, $v) = split(/\s*=\s*/, $value, 2);
-        # strip quoting
-        $v =~ s/^(['"])(.*)\1$/$2/ if defined $v;
-        $self->{ VARIABLE }->{ $variable }->{ $k } = $v;
+	# insert "<key>=<value>" data into hash 
+	my ($k, $v) = split(/\s*=\s*/, $value, 2);
+	# strip quoting
+	$v =~ s/^(['"])(.*)\1$/$2/ if defined $v;
+	$self->{ VARIABLE }->{ $variable }->{ $k } = $v;
     }
     else {
-        # set simple variable
-        $self->{ VARIABLE }->{ $variable } = $value;
+	# set simple variable
+	$self->{ VARIABLE }->{ $variable } = $value;
     }
 
 
     # call any ACTION function bound to this variable
     return &{ $self->{ ACTION }->{ $variable } }($self, $variable, $value)
-        if (exists($self->{ ACTION }->{ $variable }));
+    	if (exists($self->{ ACTION }->{ $variable }));
 
     # ...or just return 1 (ok)
     return 1;
@@ -400,7 +405,7 @@ sub varlist {
     return %set;
 }
 
-
+    
 #------------------------------------------------------------------------
 # AUTOLOAD
 #
@@ -435,25 +440,25 @@ sub AUTOLOAD {
     # method matching the attribute or flag name in lower case with 
     # a leading underscore_
     if (($attrib = $variable) =~ s/_//g) {
-        $attrib = uc $attrib;
+	$attrib = uc $attrib;
+	
+	if (exists $METHFLAGS{ $attrib }) {
+	    return $self->{ $attrib };
+	}
 
-        if (exists $METHFLAGS{ $attrib }) {
-            return $self->{ $attrib };
-        }
+	if (exists $METHVARS{ $attrib }) {
+	    # next parameter should be variable name
+	    $variable = shift;
+	    $variable = $self->_varname($variable);
 
-        if (exists $METHVARS{ $attrib }) {
-            # next parameter should be variable name
-            $variable = shift;
-            $variable = $self->_varname($variable);
+	    # check we've got a valid variable
+#	    $self->_error("$variable: no such variable or method"), 
+#		    return undef
+#		unless exists($self->{ VARIABLE }->{ $variable });
 
-            # check we've got a valid variable
-#           $self->_error("$variable: no such variable or method"), 
-#                   return undef
-#               unless exists($self->{ VARIABLE }->{ $variable });
-
-            # return attribute
-            return $self->{ $attrib }->{ $variable };
-        }
+	    # return attribute
+	    return $self->{ $attrib }->{ $variable };
+	}
     }
 
     # set a new value if a parameter was supplied or return the old one
@@ -484,18 +489,18 @@ sub _configure {
 
     foreach my $opt (keys %$cfg) {
 
-        # GLOBAL must be a hash ref
-        $opt =~ /^GLOBALS?$/i && do {
-            unless (ref($cfg->{ $opt }) eq 'HASH') {
-                $self->_error("\U$opt\E parameter is not a hash ref");
+	# GLOBAL must be a hash ref
+	$opt =~ /^GLOBALS?$/i && do {
+	    unless (ref($cfg->{ $opt }) eq 'HASH') {
+		$self->_error("\U$opt\E parameter is not a hash ref");
                 next;
             }
 
             # we check each option is ok to be in GLOBAL, but we don't do 
-            # any error checking on the values they contain (but should?).
+	    # any error checking on the values they contain (but should?).
             foreach my $global ( keys %{ $cfg->{ $opt } } )  {
 
-                # continue if the attribute is ok to be GLOBAL 
+		# continue if the attribute is ok to be GLOBAL 
                 next if ($global =~ /(^$global_ok$)/io);
 
                 $self->_error( "\U$global\E parameter cannot be GLOBAL");
@@ -504,26 +509,27 @@ sub _configure {
             next;
         };
 
-        # CASE, CREATE and PEDANTIC are stored as they are
-        $opt =~ /^CASE|CREATE|PEDANTIC$/i && do {
-            $self->{ uc $opt } = $cfg->{ $opt };
-            next;
-        };
+	
+	# CASE, CREATE and PEDANTIC are stored as they are
+	$opt =~ /^CASE|CREATE|PEDANTIC$/i && do {
+	    $self->{ uc $opt } = $cfg->{ $opt };
+	    next;
+	};
 
-        # ERROR triggers $self->_ehandler()
-        $opt =~ /^ERROR$/i && do {
-            $self->_ehandler($cfg->{ $opt });
-            next;
-        };
+	# ERROR triggers $self->_ehandler()
+	$opt =~ /^ERROR$/i && do {
+	    $self->_ehandler($cfg->{ $opt });
+	    next;
+	};
 
-        # DEBUG triggers $self->_debug()
-        $opt =~ /^DEBUG$/i && do {
-            $self->_debug($cfg->{ $opt });
-            next;
-        };
+	# DEBUG triggers $self->_debug()
+	$opt =~ /^DEBUG$/i && do {
+	    $self->_debug($cfg->{ $opt });
+	    next;
+	};
 
-        # warn about invalid options
-        $self->_error("\U$opt\E is not a valid configuration option");
+	# warn about invalid options
+	$self->_error("\U$opt\E is not a valid configuration option");
     }
 }
 
@@ -559,25 +565,26 @@ sub _varname {
 
     # get the actual name if this is an alias
     $variable = $self->{ ALIAS }->{ $variable }
-        if (exists($self->{ ALIAS }->{ $variable }));
+	if (exists($self->{ ALIAS }->{ $variable }));
 
     # if the variable doesn't exist, we can try to chop off a leading 
     # "no" and see if the remainder matches an ARGCOUNT_ZERO variable
     unless (exists($self->{ VARIABLE }->{ $variable })) {
-        # see if the variable is specified as "no<var>"
-        if ($variable =~ /^no(.*)/) {
-            # see if the real variable (minus "no") exists and it
-            # has an ARGOUNT of ARGCOUNT_NONE (or no ARGCOUNT at all)
-            my $novar = $self->_varname($1);
-            if (exists($self->{ VARIABLE }->{ $novar })
-                && ! $self->{ ARGCOUNT }->{ $novar }) {
-                # set variable name and negate value 
-                $variable = $novar;
-                $$negated = ! $$negated if defined $negated;
-            }
-        }
-    }
 
+	# see if the variable is specified as "no<var>"
+	if ($variable =~ /^no(.*)/) {
+	    # see if the real variable (minus "no") exists and it
+	    # has an ARGOUNT of ARGCOUNT_NONE (or no ARGCOUNT at all)
+	    my $novar = $self->_varname($1);
+	    if (exists($self->{ VARIABLE }->{ $novar })
+		    && ! $self->{ ARGCOUNT }->{ $novar }) {
+		# set variable name and negate value 
+		$variable = $novar;
+		$$negated = ! $$negated if defined $negated;
+	    }
+	}
+    }
+   
     # return the variable name
     $variable;
 }
@@ -599,34 +606,35 @@ sub _default {
 
     # check the variable exists
     if (exists($self->{ VARIABLE }->{ $variable })) {
-        # set variable value to the default scalar, an empty list or empty
-        # hash array, depending on its ARGCOUNT value
-        my $argcount = $self->{ ARGCOUNT }->{ $variable };
-        $argcount = AppConfig::ARGCOUNT_ONE unless defined $argcount;
 
-        if ($argcount == AppConfig::ARGCOUNT_NONE) {
-            return $self->{ VARIABLE }->{ $variable } 
-                 = $self->{ DEFAULT }->{ $variable } || 0;
-        }
-        elsif ($argcount == AppConfig::ARGCOUNT_LIST) {
+	# set variable value to the default scalar, an empty list or empty
+	# hash array, depending on its ARGCOUNT value
+	my $argcount = $self->{ ARGCOUNT }->{ $variable };
+	$argcount = AppConfig::ARGCOUNT_ONE unless defined $argcount;
+
+	if ($argcount == AppConfig::ARGCOUNT_NONE) {
+	    return $self->{ VARIABLE }->{ $variable } 
+		    = $self->{ DEFAULT }->{ $variable } || 0;
+	}
+	elsif ($argcount == AppConfig::ARGCOUNT_LIST) {
             my $deflist = $self->{ DEFAULT }->{ $variable };
             return $self->{ VARIABLE }->{ $variable } = 
                 [ ref $deflist eq 'ARRAY' ? @$deflist : ( ) ];
 
-        }
-        elsif ($argcount == AppConfig::ARGCOUNT_HASH) {
+	}
+	elsif ($argcount == AppConfig::ARGCOUNT_HASH) {
             my $defhash = $self->{ DEFAULT }->{ $variable };
             return $self->{ VARIABLE }->{ $variable } = 
-            { ref $defhash eq 'HASH' ? %$defhash : () };
-        }
-        else {
-            return $self->{ VARIABLE }->{ $variable } 
-                 = $self->{ DEFAULT }->{ $variable };
-        }
+                { ref $defhash eq 'HASH' ? %$defhash : () };
+	}
+	else {
+	    return $self->{ VARIABLE }->{ $variable } 
+		    = $self->{ DEFAULT }->{ $variable };
+	}
     }
     else {
-        $self->_error("$variable: no such variable");
-        return 0;
+	$self->_error("$variable: no such variable");
+	return 0;
     }
 }
 
@@ -681,16 +689,16 @@ sub _validate {
 
     # CODE ref
     ref($validator) eq 'CODE' && do {
-        # run the validation function and return the result
-        return &$validator($variable, $value);
+    	# run the validation function and return the result
+       	return &$validator($variable, $value);
     };
 
     # non-ref (i.e. scalar)
     ref($validator) || do {
-        # not a ref - assume it's a regex
-        return $value =~ /$validator/;
+	# not a ref - assume it's a regex
+	return $value =~ /$validator/;
     };
-
+    
     # validation failed
     return 0;
 }
@@ -712,10 +720,10 @@ sub _error {
 
     # user defined error handler?
     if (ref($self->{ EHANDLER }) eq 'CODE') {
-        &{ $self->{ EHANDLER } }($format, @_);
+	&{ $self->{ EHANDLER } }($format, @_);
     }
     else {
-        warn(sprintf("$format\n", @_));
+	warn(sprintf("$format\n", @_));
     }
 }
 
@@ -745,16 +753,16 @@ sub _ehandler {
 
     # update internal reference if a new handler vas provide
     if (defined $handler) {
-        # check this is a code reference
-        if (ref($handler) eq 'CODE') {
-            $self->{ EHANDLER } = $handler;
+	# check this is a code reference
+	if (ref($handler) eq 'CODE') {
+	    $self->{ EHANDLER } = $handler;
 
-            # DEBUG
-            print STDERR "installed new ERROR handler: $handler\n" if $DEBUG;
-        }
-        else {
-            $self->_error("ERROR handler parameter is not a code ref");
-        }
+	    # DEBUG
+	    print STDERR "installed new ERROR handler: $handler\n" if $DEBUG;
+	}
+	else {
+	    $self->_error("ERROR handler parameter is not a code ref");
+	}
     }
 
     return $previous;
@@ -804,32 +812,32 @@ sub _dump_var {
     # $var may be an alias, so we resolve the real variable name
     my $real = $self->_varname($var);
     if ($var eq $real) {
-        print STDERR "$var\n";
+    	print STDERR "$var\n";
     }
     else {
-        print STDERR "$real  ('$var' is an alias)\n";
-        $var = $real;
+    	print STDERR "$real  ('$var' is an alias)\n";
+	$var = $real;
     }
 
     # for some bizarre reason, the variable VALUE is stored in VARIABLE
     # (it made sense at some point in time)
     printf STDERR "    VALUE        => %s\n", 
-                defined($self->{ VARIABLE }->{ $var }) 
-                    ? $self->{ VARIABLE }->{ $var } 
-                    : "<undef>";
+		defined($self->{ VARIABLE }->{ $var }) 
+		    ? $self->{ VARIABLE }->{ $var } 
+		    : "<undef>";
 
     # the rest of the values can be read straight out of their hashes
     foreach my $param (qw( DEFAULT ARGCOUNT VALIDATE ACTION EXPAND )) {
-        printf STDERR "    %-12s => %s\n", $param, 
-                defined($self->{ $param }->{ $var }) 
-                    ? $self->{ $param }->{ $var } 
-                    : "<undef>";
+	printf STDERR "    %-12s => %s\n", $param, 
+		defined($self->{ $param }->{ $var }) 
+		    ? $self->{ $param }->{ $var } 
+		    : "<undef>";
     }
 
     # summarise all known aliases for this variable
     print STDERR "    ALIASES      => ", 
-            join(", ", @{ $self->{ ALIASES }->{ $var } }), "\n"
-            if defined $self->{ ALIASES }->{ $var };
+	    join(", ", @{ $self->{ ALIASES }->{ $var } }), "\n"
+	if defined $self->{ ALIASES }->{ $var };
 } 
 
 
@@ -845,23 +853,23 @@ sub _dump {
 
     print STDERR "=" x 71, "\n";
     print STDERR 
-        "Status of AppConfig::State (version $VERSION) object:\n\t$self\n";
-
+	"Status of AppConfig::State (version $VERSION) object:\n\t$self\n";
 
+    
     print STDERR "- " x 36, "\nINTERNAL STATE:\n";
     foreach (qw( CREATE CASE PEDANTIC EHANDLER ERROR )) {
-        printf STDERR "    %-12s => %s\n", $_, 
-                defined($self->{ $_ }) ? $self->{ $_ } : "<undef>";
-    }       
+	printf STDERR "    %-12s => %s\n", $_, 
+		defined($self->{ $_ }) ? $self->{ $_ } : "<undef>";
+    }	    
 
     print STDERR "- " x 36, "\nVARIABLES:\n";
     foreach $var (keys %{ $self->{ VARIABLE } }) {
-        $self->_dump_var($var);
+	$self->_dump_var($var);
     }
 
     print STDERR "- " x 36, "\n", "ALIASES:\n";
     foreach $var (keys %{ $self->{ ALIAS } }) {
-        printf("    %-12s => %s\n", $var, $self->{ ALIAS }->{ $var });
+	printf("    %-12s => %s\n", $var, $self->{ ALIAS }->{ $var });
     }
     print STDERR "=" x 72, "\n";
 } 
@@ -888,7 +896,7 @@ AppConfig::State - application configuration state
 
     $state->set("foo", 123);          # trivial set/get examples
     $state->get("foo");      
-
+    
     $state->foo();                    # shortcut variable access 
     $state->foo(456);                 # shortcut variable update 
 
@@ -912,12 +920,12 @@ appear in your Perl script:
 
 The AppConfig::State module is loaded automatically by the new()
 constructor of the AppConfig module.
-
+      
 AppConfig::State is implemented using object-oriented methods.  A 
 new AppConfig::State object is created and initialised using the 
 new() method.  This returns a reference to a new AppConfig::State 
 object.
-
+       
     my $state = AppConfig::State->new();
 
 This will create a reference to a new AppConfig::State with all 
@@ -926,8 +934,8 @@ the object by passing a reference to a hash array containing
 configuration options:
 
     $state = AppConfig::State->new( {
-        CASE      => 1,
-        ERROR     => \&my_error,
+	CASE      => 1,
+	ERROR     => \&my_error,
     } );
 
 The new() constructor of the AppConfig module automatically passes all 
@@ -961,7 +969,7 @@ variable is created.  This can be used to specify configuration file
 blocks in which variables should be created, for example:
 
     $state = AppConfig::State->new( {
-        CREATE => '^define_',
+	CREATE => '^define_',
     } );
 
 In a config file:
@@ -1021,11 +1029,11 @@ The GLOBAL option allows default values to be set for the DEFAULT, ARGCOUNT,
 EXPAND, VALIDATE and ACTION options for any subsequently defined variables.
 
     $state = AppConfig::State->new({
-        GLOBAL => {
-            DEFAULT  => '<undef>',     # default value for new vars
-            ARGCOUNT => 1,             # vars expect an argument
-            ACTION   => \&my_set_var,  # callback when vars get set
-        }
+	GLOBAL => {
+	    DEFAULT  => '<undef>',     # default value for new vars
+	    ARGCOUNT => 1,             # vars expect an argument
+	    ACTION   => \&my_set_var,  # callback when vars get set
+	}
     });
 
 Any attributes specified explicitly when a variable is defined will
@@ -1047,9 +1055,9 @@ reference to a hash array may also be passed to specify configuration
 information for the variable:
 
     $state->define("foo", {
-            DEFAULT   => 99,
-            ALIAS     => 'metavar1',
-        });
+	    DEFAULT   => 99,
+	    ALIAS     => 'metavar1',
+	});
 
 Any variable-wide GLOBAL values passed to the new() constructor in the 
 configuration hash will also be applied.  Values explicitly specified 
@@ -1065,8 +1073,8 @@ The following configuration options may be specified
 The DEFAULT value is used to initialise the variable.  
 
     $state->define("drink", {
-            DEFAULT => 'coffee',
-        });
+	    DEFAULT => 'coffee',
+	});
 
     print $state->drink();        # prints "coffee"
 
@@ -1077,20 +1085,17 @@ this variable.  A single alias should be specified as a string.  Multiple
 aliases can be specified as a reference to an array of alternatives or as 
 a string of names separated by vertical bars, '|'.  e.g.:
 
-    # either
     $state->define("name", {
-            ALIAS  => 'person',
-        });
-
-    # or
+	    ALIAS  => 'person',
+	});
+or
     $state->define("name", {
-            ALIAS => [ 'person', 'user', 'uid' ],
-        });
-
-    # or
+	    ALIAS => [ 'person', 'user', 'uid' ],
+	});
+or
     $state->define("name", {
-            ALIAS => 'person|user|uid',
-        });
+	    ALIAS => 'person|user|uid',
+	});
 
     $state->user('abw');     # equivalent to $state->name('abw');
 
@@ -1151,8 +1156,8 @@ See those two modules for more information on the format and meaning of
 these options.
 
     $state->define("name", {
-            ARGS => "=i@",
-        });
+	    ARGS => "=i@",
+	});
 
 =item EXPAND 
 
@@ -1190,9 +1195,6 @@ to the values of the relevant AppConfig::State variables.
 
 Indicates that '~' or '~uid' patterns in the string should be 
 expanded to the current users ($<), or specified user's home directory.
-In the first case, C<~> is expanded to the value of the C<HOME>
-environment variable.  In the second case, the C<getpwnam()> method
-is used if it is available on your system (which it isn't on Win32).
 
 =item EXPAND_ENV
 
@@ -1232,12 +1234,12 @@ above) then this value will be used as the default VALIDATE for each
 variable unless otherwise specified.
 
     $state->define("age", {
-            VALIDATE => '\d+',
-        });
+    	    VALIDATE => '\d+',
+	});
 
     $state->define("pin", {
-            VALIDATE => \&check_pin,
-        });
+	    VALIDATE => \&check_pin,
+	});
 
 =item ACTION
 
@@ -1255,11 +1257,11 @@ Example:
     $state->define("foo", { ACTION => \&my_notify });
 
     sub my_notify {
-        my $state = shift;
-        my $var   = shift;
-        my $val   = shift;
+	my $state = shift;
+	my $var   = shift;
+	my $val   = shift;
 
-        print "$variable set to $value";
+	print "$variable set to $value";
     }
 
     $state->foo(42);        # prints "foo set to 42"
@@ -1268,6 +1270,8 @@ Be aware that calling C<$state-E<gt>set()> to update the same variable
 from within the ACTION function will cause a recursive loop as the
 ACTION function is repeatedly called.  
 
+=item 
+
 =back
 
 =head2 DEFINING VARIABLES USING THE COMPACT FORMAT
@@ -1283,9 +1287,9 @@ after the variable name(s) and/or aliases.
 The following examples are equivalent:
 
     $state->define("foo", { 
-            ALIAS => [ 'bar', 'baz' ],
-            ARGS  => '=i',
-        });
+	    ALIAS => [ 'bar', 'baz' ],
+	    ARGS  => '=i',
+	});
 
     $state->define("foo|bar|baz=i");
 
@@ -1392,9 +1396,13 @@ debugging purposes.
 
 Andy Wardley, E<lt>abw@wardley.orgE<gt>
 
+=head1 REVISION
+
+$Revision: 1.2 $
+
 =head1 COPYRIGHT
 
-Copyright (C) 1997-2007 Andy Wardley.  All Rights Reserved.
+Copyright (C) 1997-2003 Andy Wardley.  All Rights Reserved.
 
 Copyright (C) 1997,1998 Canon Research Centre Europe Ltd.
 
