@@ -10,52 +10,35 @@
 # Copyright (C) 1997-2003 Andy Wardley.  All Rights Reserved.
 # Copyright (C) 1997,1998 Canon Research Centre Europe Ltd.
 #
-# $Id: Sys.pm,v 1.61 2004/02/04 10:11:23 abw Exp $
+# $Id: Sys.pm,v 1.2 2003/04/29 09:22:33 abw Exp $
 #
 #============================================================================
 
 package AppConfig::Sys;
+
+require 5.004;
 use strict;
-use warnings;
-use POSIX qw( getpwnam getpwuid );
-
-our $VERSION = '1.66_03';
-our ($AUTOLOAD, $OS, %CAN, %METHOD);
+use vars qw( $VERSION $AUTOLOAD $OS %CAN %METHOD);
 
+$VERSION = sprintf("%d.%02d", q$Revision: 1.2 $ =~ /(\d+)\.(\d+)/);
 
 BEGIN {
     # define the methods that may be available
-    if($^O =~ m/win32/i) {
-        $METHOD{ getpwuid } = sub { 
-            return wantarray() 
-                ? ( (undef) x 7, getlogin() )
-                : getlogin(); 
-        };
-        $METHOD{ getpwnam } = sub { 
-            die("Can't getpwnam on win32"); 
-        };
-    }
-    else
-    {
-        $METHOD{ getpwuid } = sub { 
-            getpwuid( defined $_[0] ? shift : $< ); 
-        };
-        $METHOD{ getpwnam } = sub { 
-            getpwnam( defined $_[0] ? shift : '' );
-        };
-    }
+    %METHOD = (
+	'getpwnam' => sub { getpwnam(defined $_[0] ? shift : '') },
+	'getpwuid' => sub { getpwuid(defined $_[0] ? shift : $<) },
+    );
 
     # try out each METHOD to see if it's supported on this platform;
     # it's important we do this before defining AUTOLOAD which would
     # otherwise catch the unresolved call
     foreach my $method  (keys %METHOD) {
-        eval { &{ $METHOD{ $method } }() };
+	eval { &{ $METHOD{ $method } }() };
     	$CAN{ $method } = ! $@;
     }
 }
 
 
-
 #------------------------------------------------------------------------
 # new($os)
 #
@@ -67,10 +50,10 @@ BEGIN {
 
 sub new {
     my $class = shift;
-
+    
     my $self = {
-        METHOD => \%METHOD,
-        CAN    => \%CAN,
+	METHOD => \%METHOD,
+	CAN    => \%CAN,
     };
 
     bless $self, $class;
@@ -103,23 +86,18 @@ sub AUTOLOAD {
 
     # can_method()
     if ($method =~ s/^can_//i && exists $self->{ CAN }->{ $method }) {
-        return $self->{ CAN }->{ $method };
+	return $self->{ CAN }->{ $method };
     }
     # method() 
     elsif (exists $self->{ METHOD }->{ $method }) {
-        if ($self->{ CAN }->{ $method }) {
-            return &{ $self->{ METHOD }->{ $method } }(@_);
-        }
-        else {
-            return undef;
-        }
+	return &{ $self->{ METHOD }->{ $method } }(@_);
     } 
     # variable
     elsif (exists $self->{ uc $method }) {
-        return $self->{ uc $method };
+	return $self->{ uc $method };
     }
     else {
-        warn("AppConfig::Sys->", $method, "(): no such method or variable\n");
+	warn("AppConfig::Sys->", $method, "(): no such method or variable\n");
     }
 
     return undef;
@@ -155,27 +133,27 @@ sub _configure {
 	    $os = $Config::Config{'osname'};
 	}
     }
-    if ($os =~ /win32/i) {
-        $os = 'WINDOWS';
+    if ($os =~ /Win/i) {
+	$os = 'WINDOWS';
     } elsif ($os =~ /vms/i) {
-        $os = 'VMS';
-    } elsif ($os =~ /mac/i) {
-        $os = 'MACINTOSH';
+	$os = 'VMS';
+    } elsif ($os =~ /Mac/i) {
+	$os = 'MACINTOSH';
     } elsif ($os =~ /os2/i) {
-        $os = 'OS2';
+	$os = 'OS2';
     } else {
-        $os = 'UNIX';
+	$os = 'UNIX';
     }
 
 
     # The path separator is a slash, backslash or semicolon, depending
     # on the platform.
     my $ps = {
-        UNIX      => '/',
-        OS2       => '\\',
-        WINDOWS   => '\\',
-        MACINTOSH => ':',
-        VMS       => '\\'
+	UNIX      => '/',
+	OS2       => '\\',
+	WINDOWS   => '\\',
+	MACINTOSH => ':',
+	VMS       => '\\'
     }->{ $os };
     #
     # Thanks Lincoln!
@@ -202,8 +180,8 @@ sub _dump {
     print "      Path Separator : ", $self->{ PATHSEP }, "\n";
     print "   Available methods :\n";
     foreach my $can (keys %{ $self->{ CAN } }) {
-        printf "%20s : ", $can;
-        print  $self->{ CAN }->{ $can } ? "yes" : "no", "\n";
+	printf "%20s : ", $can;
+	print  $self->{ CAN }->{ $can } ? "yes" : "no", "\n";
     }
     print "=" x 71, "\n";
 }
@@ -214,8 +192,6 @@ sub _dump {
 
 __END__
 
-=pod
-
 =head1 NAME
 
 AppConfig::Sys - Perl5 module defining platform-specific information and methods for other AppConfig::* modules.
@@ -237,7 +213,7 @@ AppConfig::Sys is distributed as part of the AppConfig bundle.
 
 =head1 DESCRIPTION
 
-=head2 USING THE AppConfig::Sys MODULE
+=head2 USING THE AppConfig::State MODULE
 
 To import and use the AppConfig::Sys module the following line should
 appear in your Perl script:
@@ -259,7 +235,7 @@ detection:
     $unix_sys = AppConfig::Sys->new("Unix");
 
 Alternatively, the package variable $AppConfig::Sys::OS can be set to an
-operating system name.  The valid operating system names are: Win32, VMS,
+operating system name.  The valid operating system names are: Win, VMS,
 Mac, OS2 and Unix.  They are not case-specific.
 
 =head2 AppConfig::Sys METHODS
@@ -280,15 +256,19 @@ Calls the system function getpwuid() if available and returns the result.
 Returns undef if not available.  The can_getpwuid() method can be called to
 determine if this function is available.
 
-=back
+=item 
 
 =head1 AUTHOR
 
 Andy Wardley, E<lt>abw@wardley.orgE<gt>
 
+=head1 REVISION
+
+$Revision: 1.2 $
+
 =head1 COPYRIGHT
 
-Copyright (C) 1997-2007 Andy Wardley.  All Rights Reserved.
+Copyright (C) 1997-2003 Andy Wardley.  All Rights Reserved.
 
 Copyright (C) 1997,1998 Canon Research Centre Europe Ltd.
 
